#include <ESP8266WiFi.h>          
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
//adres czujnika BME280 to 0x76 trzeba to zmienić w pliku Adafruit_BME280.h
#include <ESP8266WebServer.h>
#include <DNSServer.h>
#include <WiFiManager.h>  
#include <SoftwareSerial.h>

#define TRIGGER_PIN 14
#define N 31

Adafruit_BME280 bme; // I2C

unsigned long delayTime;

WiFiServer serverHTTP(80);
WiFiServer serverRX(50000);

SoftwareSerial mySerial(13, 2); // RX, TX

int count = 0;
unsigned char bufor [N];
int PM25 = 0, PM10 = 0, PM1 = 0;

void setup() {
  Serial.begin(115200);
  Serial.println("\n SMOG TX\n");

  pinMode(TRIGGER_PIN, INPUT);
  if ( digitalRead(TRIGGER_PIN) == LOW ) {
    //-----------------WiFiManager---------------
    Serial.println("\n WIFI CONFIG MODE");
    WiFiManager wifiManager;
    if (!wifiManager.startConfigPortal("SMOG_TX_SETUP")) {
      Serial.println("failed to connect and hit timeout");
    }
    Serial.print("Connected! Device IP is: ");
    Serial.print(WiFi.localIP());
    Serial.println("");
  }
  else  {
    //----------------regular run-----------------
    bool status;
    mySerial.begin(9600); //serial do czujnika
    status = bme.begin();
    if (!status) {
        Serial.println("Could not find a valid BME280 sensor, check wiring!");
        while (1);
    }
    delayTime = 1000;
    Serial.println();
    delay(100); 
    Serial.println("\n REGULAR RUN MODE");
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
    }
    Serial.print("\nConnected! Device IP is: ");
    Serial.print(WiFi.localIP());
    Serial.println("");
    serverHTTP.begin();
    Serial.println("HTTP Server started");
    serverRX.begin();
    Serial.println("RX Server started");
  }
}

//------------------------------loop---------------------------------
void loop() {
  if(mySerial.find(0x42))    
    mySerial.readBytes(bufor,N);
  if(bufor[0] == 0x4d) {
    PM25=wartoscPM25(bufor);
    PM10=wartoscPM10(bufor);  
    PM1=wartoscPM1(bufor);
  }
  //Serial.printf("PM1=%d ug/m3\n\r",PM1);
  //Serial.printf("PM2.5=%d ug/m3\n\r",PM25);
  //Serial.printf("PM10=%d ug/m3\n\r",PM10);
  WiFiClient clientHTTP = serverHTTP.available();
  if (clientHTTP) {
    Serial.println("new client HTTP");
    while(!clientHTTP.available()){
      delay(1);
    }
    String req = clientHTTP.readStringUntil('\r');
    Serial.println(req);
    clientHTTP.flush();
    int val =1;
    String s = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<!DOCTYPE HTML>\r\n<html>\r\n<head>\r\n<title>SMOG TX</title>\r\n<meta http-equiv=\"refresh\" content=\"5\">\r\n</head>\r\n<body>\r\n<b>\r\n";
    s += "SMOG TX<br><br>\r\n";
    s += "Temperatura = ";
    s += (bme.readTemperature() - 4);
    s += " *C<br>\r\n";

    s += "Cisnienie = ";
    s += (bme.readPressure() / 100.0F);
    s += " hPa<br>\r\n";
 
    s += "Wilgotnosc = ";
    s += bme.readHumidity();
    s += " %<br>\r\n";

    s += "Jakosc powietrza: <br>\r\n";
    //----------PM2.5---------------
    s += "PM2.5 = ";
    s += "<FONT style=\"BACKGROUND-COLOR: ";
    if(PM25<=12)
      s+= "green\">";
    if(PM25>=13&&PM25<37)
      s+= "lime\">";
    if(PM25>=37&&PM25<60)
      s+= "yellow\">";
    if(PM25>=61&&PM25<84)
      s+= "orange\">";
    if(PM25>=85&&PM25<120)
      s+= "red\">";
    if(PM25>=120)
      s+= "gray\">";
    
    s += PM25;
    s += " ug/m3";
    s +="</FONT>";

    s += " - ";

    if(PM25<=12)
      s+= "SWIETNA";
    if(PM25>=13&&PM25<37)
      s+= "DOBRA";
    if(PM25>=37&&PM25<60)
      s+= "SREDNIA";
    if(PM25>=61&&PM25<84)
      s+= "NIEZDROWA";
    if(PM25>=85&&PM25<120)
      s+= "ZLA"; 
    if(PM25>120)
      s+= "FATALNA";
    s += "<br>\r\n";
    //----------PM10----------
    s += "PM10 = ";
    s += "<FONT style=\"BACKGROUND-COLOR: ";
  
    if(PM10<=20)
      s+= "green\">";
    if(PM10>=21&&PM10<60)
      s+= "lime\">";
    if(PM10>=61&&PM10<100)
      s+= "yellow\">";
    if(PM10>=101&&PM10<140)
      s+= "orange\">";
    if(PM10>=141&&PM10<200)
      s+= "red\">"; 
    if(PM10>=200)
      s+= "gray\">";
 
    s += PM10;
    s += " ug/m3";
    s+="</FONT>";
    s += " - ";
  
    if(PM10<=20)
      s+= "SWIETNA";
    if(PM10>=21&&PM10<60)
      s+= "DOBRA";
    if(PM10>=61&&PM10<100)
      s+= "SREDNIA";
    if(PM10>=101&&PM10<140)
      s+= "NIEZDROWA";
    if(PM10>=141&&PM10<200)
      s+= "ZLA";
    if(PM10>200)
      s+= "FATALNA"; 
    s += "<br>\r\n";
    //---------PM1----------
    s += "PM1 = ";
    s += PM1;
    s += " ug/m3";
    s += "<br>\r\n";
    //--------koniec strony--------
    s += "</b>\r\n</body>\r\n</html>\r\n";
    
    clientHTTP.print(s);
    clientHTTP.flush();
    delay(1);
    Serial.println("HTTP Client disonnected");
    // The client will actually be disconnected 
    // when the function returns and 'client' object is detroyed
  }
//---------------------client RX----------------------------
  WiFiClient clientRX = serverRX.available();
  if (clientRX) {
    Serial.println("new client RX");
    while(!clientRX.available()){
      delay(1);
    }
    String reqRX = clientRX.readStringUntil('\r');
    Serial.println(reqRX);
    clientRX.flush();
    String s2 = "PM25=";
    s2 += PM25;
    s2 += "\n\r";
    s2 += "PM10=";
    s2 += PM10;
    s2 += "\n\r";
    clientRX.print(s2);
    clientRX.flush();
    delay(1);
    Serial.println("RX Client disonnected");
  }
}


int wartoscPM25(unsigned char *buf) // generacja wartosci PM2,5
{
  int PM25v;
  PM25v=((buf[11]<<8) + buf[12]);  
  return PM25v;
}
 
int wartoscPM10(unsigned char *buf) // generacja wartosci PM10
{
  int PM10v;
  PM10v=((buf[13]<<8) + buf[14]); 
  return PM10v;
}

int wartoscPM1(unsigned char *buf) // generacja wartosci PM10
{
  int PM1v;
  PM1v=((buf[9]<<8) + buf[10]); 
  return PM1v;
}
